import chutney.colors as colors

from .weather import weatherFile


class WeatherDisplayer:
    def __init__(self, symbolDisplayer, topRow):
        self.symbolDisplayer = symbolDisplayer
        self.topRow = topRow
        self.twoDigitStartColumn = 5
        self.oneDigitStartColumn = 7
        self.digitWidth = 4
        self.currentTemperature = ''
        self.color = colors.ORANGE

    def showWeather(self):
        temperature = self.getTemperature()

        if temperature != self.currentTemperature:
            self.currentTemperature = temperature
            self.symbolDisplayer.clearRow(self.topRow)

            if self.isTemperatureValid(temperature):
                self.updateColor(temperature)
                self.showTemperature(temperature)

    def getTemperature(self):
        try:
            with open(weatherFile) as file:
                return file.readline().strip()
        except Exception as e:
            print(e, flush=True)
            return 'error'

    def isTemperatureValid(self, temperature):
        return not (temperature == 'error' or temperature == 'init' or temperature == 'halted')

    def updateColor(self, temperature):
        if int(temperature) > 0:
            self.color = colors.ORANGE
        else:
            self.color = colors.BLUE

    def showTemperature(self, temperature):
        column = self.getStartColumn(temperature)

        for c in list(temperature):
            if c == '-':
                offset = 2 if self.startsWithOne(temperature) else 3
                self.showNegative(column - offset)
            else:
                self.showDigit(column, c)
                column = column + self.digitWidth

        self.showDegree(column)

    def getStartColumn(self, temperature):
        offset = 1 if self.startsWithOne(temperature) else 0
        startColumn = self.oneDigitStartColumn if self.isOneDigit(
            temperature) else self.twoDigitStartColumn

        return startColumn - offset

    def startsWithOne(self, temperature):
        absoluteTemperature = abs(int(temperature))

        return absoluteTemperature == 1 or absoluteTemperature//10 == 1

    def isOneDigit(self, temperature):
        return abs(int(temperature))//10 == 0

    def showNegative(self, column):
        self.symbolDisplayer.displayNegative(
            x=column,
            y=self.topRow,
            color=self.color
        )

    def showDigit(self, column, digit):
        self.symbolDisplayer.displayDigit(
            x=column,
            y=self.topRow,
            digit=digit,
            color=self.color
        )

    def showDegree(self, column):
        self.symbolDisplayer.displayDegree(
            x=column,
            y=self.topRow,
            color=self.color
        )
